# -*- coding: utf-8 -*-
"""
Porpose: audiomass __init__ file that holds package information.
Writer: Gianluca Pernigoto <jeanlucperni@gmail.com>
Copyright: (c) 2015/2020 Gianluca Pernigoto <jeanlucperni@gmail.com>
license: GPL3
Rev: July.25.2020, Dec 08 2021
Code checker: flake8, pylint
"""

__author__ = ('Gianluca Pernigotto', '(aka jeanslack)')
__mail__ = '<jeanlucperni@gmail.com>'
__copyright__ = '© 2013-2021'
__version__ = '0.9.5'
__release__ = 'December 16 2021'
__rls_name__ = "Audiomass"
__prg_name__ = "audiomass"
__url__ = "https://github.com/jeanslack/audiomass"
__short_descript__ = 'Wrapper for multiple audio conversion libraries. '
__long_descript__ = """
**Audiomass** is a command line audio wrapper of the Flac, Lame, Vorbis-tools,
Monkey's Audio, Shntool and FFmpeg libraries. It supports conversions of
different audio formats at a time and the ability to convert even groups of
files in a directory, saving the output in a specific folder.
"""

__try__ = "type 'audiomass -h' for help or, more detailed, run 'man audiomass'"
__license__ = (f"""
    Copyright {__copyright__} - {__author__[0]} {__author__[1]}
    Mail: {__mail__}

    Audiomass is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or (at
    your option) any later version.

    Audiomass is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with audiomass; if not, see <http://www.gnu.org/licenses/>
    or write to the Free Software Foundation, Inc., 51 Franklin St,
    Fifth Floor, Boston, MA  02110-1301 USA
    ==============================================================
    """)

__short_license__ = "'Gnu GPL3 (Gnu Public License)"



# -*- coding: utf-8 -*-
"""
Name:         batch_conversion.py (module)
Porpose:      Handles the audio data files for batch conversions.
Writer:       Gianluca Pernigoto <jeanlucperni@gmail.com>
Copyright:    (c) Gianluca Pernigoto <jeanlucperni@gmail.com>
license:      GPL3
Rev:          Nov 27 2017, Dec.15 2017, Aug.10 2019, Dec 20 2021
Code checker: flake8, pylint
"""
import sys
import os
from collections import Counter
from audiomass.datastrings import msgdebug, msgcolor, msgcustom, msgend
from audiomass.comparisions import supported_formats
from audiomass.utils import (show_format_menu,
                             get_codec_data,
                             build_cmd,
                             run_subprocess)


class BatchConvert():
    """
    The goal of this class is to obtain a catalog of one
    or more audio files, even with different formats and
    paths, and associate them with the data required for
    conversion into other different audio formats,
    all in a convenient data structure for processing.

    USE:
        conv = BatchConvert([file.wav, file.flac, ...], out=None)
        conv.prompt_to_format()
        conv.command_builder()
        conv.end_check()

    NOTE: the ``out`` default argument can accept the
          path of an output folder.
          the ``end_check`` method is optional but useful
          for printing debug messages and the final result.
    """
    def __init__(self, f_list, outdir):
        """
        Evaluates formats compatibility of the imported files.
        This method is responsible to defining `self.filecatalog`
        attribute in order to group the files and data as values
        of a format key, example:

        {'wav': {'files': [file1.wav, file2.wav] 'data_codec': data, ...},
         'flac': {'files': [file3.flac], 'data_codec': ...},
         'mp3': {'files': [file4.mp3, file5.mp3], 'data_codec': ...}
         }

        """
        self.filecatalog = {}
        self.outdir = outdir
        self.not_processed = []
        self.processed = []
        self.warnings = []
        self.info = []
        self.errors = []
        f_list = self.remove_duplicates_from_list(f_list)
        unsupported = []

        # populate self.filecatalog:
        for infile in f_list:
            ext = os.path.splitext(infile)[1].replace(".", "")

            for frmt in supported_formats().values():
                if ext in frmt:
                    if frmt[1] not in self.filecatalog:  # create the key...
                        self.filecatalog[frmt[1]] = {'files': [],
                                                     'index': None}
                    # ...and
                    unsupported.append(infile)
                    self.filecatalog[frmt[1]]['files'].append(infile)
                    self.filecatalog[frmt[1]]['index'] = frmt[2]

        # make unsupported debug
        unsupported = [val for val in f_list if val not in unsupported]
        for unsup in unsupported:
            if unsup:
                self.warnings.append(f"Unsupported: '{unsup}' ..skip")
        # if not files...
        if not self.filecatalog:
            self.errors.append('...No audio files to convert, exit!')
    # ------------------------------------------------------------------#

    def remove_duplicates_from_list(self, f_list):
        """
        Clean-up list contaminated by possible duplicate files
        using set()
        """
        val = 0
        for key, val in Counter(f_list).items():
            if val > 1:
                self.info.append(f"duplicate: '{key}' ..removing")

        # removal of any duplicates in the list using set()
        return list(set(f_list)) if val > 1 else f_list
    # ------------------------------------------------------------------#

    def prompt_to_output_format(self):
        """
        Prompt to set codec and output audio format.
        """
        for input_format, val in list(self.filecatalog.items()):
            msgcustom(f"\n\033[1mConvert the '\033[32;1m"
                      f"{input_format}\033[0m'format to:\033[0m")

            show_format_menu(val['index'])  # show text menu before prompt
            while True:
                output_select = input("Type a format number among those "
                                      "available, and press the Enter key > ")
                if output_select in ('a', 'A'):
                    msgend(abort=True)
                    sys.exit()
                codec = get_codec_data(input_format, output_select)

                if codec is None:
                    msgdebug(err=f"Invalid option '{output_select}'")
                    continue
                break

            self.filecatalog[input_format]['codec'] = codec[0]
            self.filecatalog[input_format]['format'] = codec[4]

            self.prompt_to_bitrate(codec, input_format)
    # ------------------------------------------------------------------#

    def prompt_to_bitrate(self, codec, input_format):
        """
        Prompt to set the audio bitrate
        """
        if codec[1] is None:  # None bitrate
            bitrate = ''
        else:
            msgcustom(codec[2])  # show menu

            while True:
                level = input(codec[3])  # show text menu before prompt
                if level in codec[1]:
                    bitrate = codec[1][level]
                else:
                    msgdebug(err=f"Invalid option '{level}'")
                    continue
                break

        self.filecatalog[input_format]['bitrate'] = bitrate
    # ------------------------------------------------------------------#

    def command_builder(self):
        """
        Build the command and pass it to `run_subprocess` function
        """
        count = 0
        for key, val in self.filecatalog.items():
            for fname in val['files']:
                inputfile = os.path.basename(fname)
                file_name = os.path.splitext(inputfile)[0]
                count += 1
                if self.outdir is None:
                    self.outdir = os.path.dirname(fname)
                # rendo portabili i pathnames:
                norm = os.path.join(self.outdir,
                                    f"{file_name}.{val['format']}")
                if os.path.exists(norm):
                    self.warnings.append(f"Already exists > '{norm}' ..skip")
                    self.not_processed.append(fname)
                    continue
                command = build_cmd(val['codec'],
                                    val['bitrate'],
                                    fname,
                                    norm
                                    )
                msgcolor(head='\n', green2=(f"|{str(count)}| "
                                            # f"{val['format']} "
                                            f"{key} "
                                            f"Output:"), tail=f" >> '{norm}'")
                self.processed.append(norm)
                run_subprocess(command)
    # ------------------------------------------------------------------#

    def end_check(self):
        """
        Print debug messages at the end of the tasks
        """
        if self.info:
            for msg in self.info:
                msgdebug(info=msg)

        if self.warnings:
            for msg in self.warnings:
                msgdebug(warn=msg)

        if self.errors:
            for msg in self.errors:
                msgdebug(err=msg)

        if self.not_processed:
            msgcolor(orange="\nInput files NOT converted:")
            for list1 in self.not_processed:
                msgcustom(list1)

        if self.processed:
            msgcolor(green="\nConverted successfully:")
            for list2 in self.processed:
                msgcustom(list2)
        msgend(done=True)



# -*- coding: utf-8 -*-
"""
Name:         comparisions.py (module)
Porpose:      module for output strings commands and formats comparisions
Writer:       Gianluca Pernigoto <jeanlucperni@gmail.com>
Copyright:    (c) 2015/2016 Gianluca Pernigoto <jeanlucperni@gmail.com>
license:      GPL3
Rev:          Dec 08 2021
Code checker: flake8, pylint
"""


def text_menu():
    """
    Returns a descriptive list of string items to
    realize a text menu in the audio formats selection

    """
    return [
      "------------",
      " \033[1m 1 \033[0m> Wav   (WAVEform audio format, PCM uncompresed)",
      " \033[1m 2 \033[0m> Aiff  (Apple Interchange File Format)",
      " \033[1m 3 \033[0m> Flac  (Free Lossless Audio Codecs)",
      " \033[1m 4 \033[0m> Ape   (Monkey's audio)",
      " \033[1m 5 \033[0m> Mp3   (MPEG-1 Audio Layer 3)",
      " \033[1m 6 \033[0m> Ogg   (ogg-vorbis lossy format)",
      "------------",
      " \033[41;37;1m A \033[0m \033[1m..ABORT\033[0m",
      "------------"]


def supported_formats():
    """
    Returns a numbered collection of currently supported
    formats with indexing list for the `text_menu()` function.
    Given a format, enable to iterates over its indexes in order to
    exclude the items indexed on the list returned by `text_menu()`
    function, example:

        menu = text_menu()
        for formats in supported_formats().values():
            if 'flac' in formats[0:2]:
                setmenu = [menu[i] for i in range(len(menu))
                           if i not in set(formats[2])
                           ]
                for items in setmenu:
                    print(items)
            else:
                None

    The example above removes index(3) from the text_menu()

    """
    return {1: ('WAV', 'wav', [1]),
            2: ('AIFF', 'aiff', [2, 4]),
            3: ('FLAC', 'flac', [3]),
            4: ('APE', 'ape', [3, 4]),
            5: ('MP3', 'mp3', [3, 4, 5, 6]),
            6: ('OGG', 'ogg', [3, 4, 5, 6]),
            }


def comparing(pair):
    """
    Returns the required values for each pair of audio formats.
    Accept one only string argument in this form: 'format > format'.
    You can add or remove new formats modules from here.

    USE:
        comparing('wav > mp3')
        to get data to convert from wav format to mp3
    """

    # ------------------------------  BITRATES MENU:
    flac_menu = (
                "\n\033[1m"
                "Choose the quality for Flac format:\033[0m\n"
                "  0 = low compression > more access space > better quality\n"
                "  1\n"
                "  2\n"
                "  3\n"
                "  4 = Great quality/compression ratio\n"
                "  5 = default\n"
                "  6\n"
                "  7\n"
                "  8 = maximum compression > less space > lower quality\n"
                )

    ape_menu = ("\n\033[1m"
                "Choose the quality for Ape format:\033[0m\n"
                "  1    -c1000 = Fast (Best quality)\n"
                "  2    -c2000 = Normal\n"
                "  3    -c3000 = High\n"
                "  4    -c4000 = Extra\n"
                "  5    -c5000 = Insane\033[0m\n"
                )

    mp3_menu = ("\n\033[1m"
                "Choose the quality and bit-rate for MP3 format:\033[0m\n"
                "  0     medium    >  VBR 92 kbit\n"
                "  1     standard  >  VBR 112 kbit/s\n"
                "  2     extreme   >  VBR 150 kbit/s\n"
                "  3     insane    >  CBR 320 kbit/s\n\n"
                )

    ffmpeg_mp3_menu = (
                "\n\033[1m"
                "Choose the quality and bitrate for MP3 format:\033[0m\n"
                "  0     >  VBR 128 kbit\n"
                "  1     >  VBR 160 kbit/s\n"
                "  2     >  VBR 192 kbit/s\n"
                "  3     >  VBR 260 kbit/s\n"
                "  4     >  CBR 320 kbit/s\n"
                )

    ogg_menu = ("\n\033[1m"
                "Choose the quality and bit-rate for OGG format:\033[0m\n"
                "  1    >   80 kbit        |     6   >   192 kbit/s\n"
                "  2    >   92 kbit/s      |     7   >   200 kbit/s\n"
                "  3    >   100 kbit/s     |     8   >   260 kbit/s\n"
                "  4    >   128 kbit/s     |     9   >   320 kbit/s\n"
                "  5    >   134 kbit/s     |     10  >   520 kbit/s\n"
                )

    ffmpeg_ogg_menu = (
                "\n\033[1m"
                "Choose the quality and bit-rate for OGG format:\033[0m\n"
                "  0    >  VBR 128 kbit\n"
                "  1    >  VBR 160 kbit/s\n"
                "  2    >  VBR 192 kbit/s\n"
                "  3    >  VBR 260 kbit/s\n"
                "  4    >  CBR 320 kbit/s\n"
                )

    # ----------------------------- BITRATES LEVELS:

    flac_opt_comp = {"0": "-0", "1": "-1", "2": "-2", "3": "-3",
                     "4": "-4", "5": "-5", "6": "-6", "7": "-7",
                     "8": "-8"
                     }
    ape_opt_comp = {"1": "-c1000", "2": "-c2000", "3": "-c3000",
                    "4": "-c4000", "5": "-c5000"
                    }
    mp3_opt_comp = {"0": "--preset medium", "1": "--preset standard",
                    "2": "--preset extreme", "3": "--preset insane"
                    }
    ffmpeg_mp3_opt_comp = {"0": "-acodec libmp3lame -b:a 128k -ar 44100",
                           "1": "-acodec libmp3lame -b:a 160k -ar 44100",
                           "2": "-acodec libmp3lame -b:a 192k -ar 44100",
                           "3": "-acodec libmp3lame -b:a 260k -ar 44100",
                           "4": "-acodec libmp3lame -b:a 320k -ar 44100",
                           }
    ogg_opt_comp = {"1": "-q 1", "2": "-q 2", "3": "-q 3", "4": "-q 4",
                    "5": "-q 5", "6": "-q 6", "7": "-q 7", "8": "-q 8",
                    "9": "-q 9", "10": "-q 10"
                    }
    ffmpeg_ogg_opt_comp = {"0": "-vn -acodec libvorbis -ar 44100 -ab 128k",
                           "1": "-vn -acodec libvorbis -ar 44100 -ab 160k",
                           "2": "-vn -acodec libvorbis -ar 44100 -ab 192k",
                           "3": "-vn -acodec libvorbis -ar 44100 -ab 260k",
                           "4": "-vn -acodec libvorbis -ar 44100 -ab 320k",
                           }

    # -------------------------------  PAIRING FORMATS:

    object_assignment = {
        'wav > aiff': ('shntool', None, None, None, 'aiff'),
        'aiff > wav': ('shntool', None, None, None, 'wav'),
        # 'aiff > ape': ('shntool', None, None, None, 'ape'),
        'flac > wav': ('shntool', None, None, None, 'wav'),
        'flac > aiff': ('shntool', None, None, None, 'aiff'),
        'flac > ape': ('shntool', None, None, None, 'ape'),
        'ape > wav': ('shntool', None, None, None, 'wav'),
        'ape > aiff': ('shntool', None, None, None, 'aiff'),
        # 'ape > flac': ('shntool', None, None, None, 'flac'),

        'wav > flac': ('flac', flac_opt_comp, flac_menu, 'Type the '
                       'compression level in digits 0 to 8, and press '
                       'enter key > ', 'flac'
                       ),
        'aiff > flac': ('flac', flac_opt_comp, flac_menu, 'Type the '
                        'compression level in digits 0 to 8, and press '
                        'enter key > ', 'flac'
                        ),
        'wav > mp3': ('lame', mp3_opt_comp, mp3_menu, 'Type the compression '
                      'level in digits 0 to 3, and press enter key > ', 'mp3'
                      ),
        'aiff > mp3': ('lame', mp3_opt_comp, mp3_menu, 'Type the compression '
                       'level in digits 0 to 3, and press enter key > ',
                       'mp3'
                       ),
        'mp3 > wav': ('lame --decode', None, None, None, 'wav'),
        'wav > ogg': ('oggenc', ogg_opt_comp, ogg_menu, 'Type the compression '
                      'level in digits 1 to 10, and press enter key > ',
                      'ogg'
                      ),
        'aiff > ogg': ('oggenc', ogg_opt_comp, ogg_menu, 'Type the '
                       'compression level in digits 1 to 10, and press '
                       'enter key > ', 'ogg'
                       ),
        'flac > ogg': ('oggenc', ogg_opt_comp, ogg_menu, 'Type the '
                       'compression level in digits 1 to 10, and press '
                       'enter key > ', 'ogg'
                       ),
        'ogg > wav': ('oggdec', None, None, None, 'wav'),
        'wav > ape': ('mac', ape_opt_comp, ape_menu, 'Type the compression '
                      'level in digits 1 to 5, and press enter key > ',
                      'ape'
                      ),
        'flac > mp3': ('ffmpeg', ffmpeg_mp3_opt_comp, ffmpeg_mp3_menu,
                       'Type the compression level in digits 0 to 4, '
                       'and press enter key > ',
                       'mp3'
                       ),
        'ape > mp3': ('ffmpeg', ffmpeg_mp3_opt_comp, ffmpeg_mp3_menu,
                      'Type the compression level in digits 0 to 4, '
                      'and press enter key > ',
                      'mp3'
                      ),
        'ape > ogg': ('ffmpeg', ffmpeg_ogg_opt_comp, ffmpeg_ogg_menu,
                      'Type the compression level in digits 0 to 4, '
                      'and press enter key > ',
                      'ogg'
                      ),
        'mp3 > aiff': ('ffmpeg', None, None, None, 'aiff'),
        'ogg > aiff': ('ffmpeg', None, None, None, 'aiff'),
        }
    return object_assignment.get(pair)



# -*- coding: utf-8 -*-
"""
Name:         dir_conversion.py (module)
Porpose:      Handles the audio files conversions into specified folder.
Writer:       Gianluca Pernigoto <jeanlucperni@gmail.com>
Copyright:    (c) Gianluca Pernigoto <jeanlucperni@gmail.com>
license:      GPL3
Rev:          Nov. 2017, Dec 08 2021
Code checker: flake8,
              pylint --ignore R0913
"""
import glob
import sys
import os
from audiomass.comparisions import supported_formats
from audiomass.comparisions import text_menu
from audiomass.datastrings import msgdebug, msgcolor, msgcustom, msgend
from audiomass.utils import (show_format_menu,
                             get_codec_data,
                             build_cmd,
                             run_subprocess)


class DirConvert():
    """
    Convert all audio files with a certain format and placed
    in a given folder.

    USE:
        conv = BatchConvert([file.wav, file.flac, ...], out=None)
        conv.prompt_to_format()
        conv.command_builder()
        conv.end_check()

    NOTE: the ``out`` default argument can accept the
          path of an output folder.
          the ``end_check`` method is optional but useful
          for printing debug messages and the final result.
    """
    def __init__(self, path_i, path_o):
        """
        Print a text menu with a list of available formats
        and wait for user input. The input value given by the
        user represents the format of the files to be converted
        in a given folder .
        """
        # (too-many-instance-attributes) (FIXME)
        self.inputdir = path_i
        self.outputdir = path_o
        self.input_format = None
        self.output_format = None
        self.index = None
        self.codec = None
        self.bitrate = ''
        self.not_processed = []
        self.processed = []
        self.warnings = []
        self.info = []
        self.errors = []

        msgcustom("\n\033[1mWhat is the files format to convert?\033[0m")
        for inputformat in text_menu():
            msgcustom(f"{inputformat}")  # show menu

        while True:
            input_selection = input("Type a format number among those "
                                    "available, and press the Enter key > ")

            if input_selection in ('a', 'A'):
                msgend(abort=True)
                sys.exit()

            try:
                selection = int(input_selection)
            except ValueError:
                msgdebug(err=(f"Invalid option '{input_selection}'"))
                continue

            if supported_formats().get(selection):
                self.input_format = supported_formats().get(selection)[1]
                self.index = supported_formats().get(selection)[2]
            else:
                msgdebug(err=(f"Invalid option '{input_selection}'"))
                continue
            break
    # ---------------------------------------------------------------#

    def prompt_to_output_format(self):
        """
        Prompt to get codec data and set the output audio format
        """
        msgcustom(f"\n\033[1mConvert the '\033[32;1m"
                  f"{self.input_format}\033[0m' format to:\033[0m")
        show_format_menu(self.index)  # show text menu before prompt

        while True:
            output_select = input("Type a format number among those "
                                  "available, and press the Enter key > ")
            if output_select in ('a', 'A'):
                msgend(abort=True)
                sys.exit()
            data_codec = get_codec_data(self.input_format, output_select)

            if data_codec is None:
                msgdebug(err=f"Invalid option '{output_select}'")
                continue
            break

        self.codec = data_codec[0]
        self.output_format = data_codec[4]

        self.prompt_to_bitrate(data_codec)
    # ---------------------------------------------------------------#

    def prompt_to_bitrate(self, data_codec):
        """
        Prompt to set the audio bitrate
        """
        if data_codec[1] is not None:  # None bitrate
            msgcustom(data_codec[2])  # show menu

            while True:
                level = input(data_codec[3])  # show text menu before prompt
                if level in data_codec[1]:
                    self.bitrate = data_codec[1][level]
                else:
                    msgdebug(err=f"Invalid option '{level}'")
                    continue
                break
    # ---------------------------------------------------------------#

    def command_builder(self):
        """
        Build the command and pass it to `run_subprocess` function
        """
        count = 0
        pname = None
        for suffix in [self.input_format, self.input_format.upper()]:
            # itero su nomi formato upper-case e lower-case
            for pname in glob.glob(os.path.join(self.inputdir, f"*.{suffix}")):
                stream_i = os.path.basename(pname)
                file_name = os.path.splitext(stream_i)[0]
                count += 1
                if self.outputdir is None:
                    # prendo lo stesso input path
                    self.outputdir = os.path.dirname(pname)

                fname = os.path.join(self.outputdir,
                                     f'{file_name}.{self.output_format}')
                if os.path.exists(fname):
                    self.warnings.append(f"Already exists > '{fname}' ..skip")
                    self.not_processed.append(pname)
                    continue

                command = build_cmd(self.codec, self.bitrate, pname, fname)
                msgcolor(head='\n', green2=(f"|{str(count)}| "
                                            f"{self.output_format} "
                                            f"Output:"), tail=f" >> '{fname}'")

                self.processed.append(fname)
                run_subprocess(command)

        if not pname:
            self.errors.append(f"No files found in '{self.input_format}' "
                               f"format!")
    # ---------------------------------------------------------------#

    def end_check(self):
        """
        Print debug messages at the end of the tasks
        """
        if self.info:
            for msg in self.info:
                msgdebug(info=msg)

        if self.warnings:
            for msg in self.warnings:
                msgdebug(warn=msg)

        if self.errors:
            for msg in self.errors:
                msgdebug(err=msg)

        if self.not_processed:
            msgcolor(orange="\nInput files NOT converted:")
            for list1 in self.not_processed:
                msgcustom(list1)

        if self.processed:
            msgcolor(green="\nConverted successfully:")
            for list2 in self.processed:
                msgcustom(list2)

        msgend(done=True)



# -*- coding: utf-8 -*-
"""
Name: cli.py
Porpose:  Main args parser for audiomass script
Writer: Gianluca Pernigoto <jeanlucperni@gmail.com>
Copyright: (c) 2015/2021 Gianluca Pernigoto <jeanlucperni@gmail.com>
license: GPL3
Rev: dec 9 2017, Aug 8 2019, Dec 09 2021, Dec 16 2021
Code checker: flake8, pylint
"""
import sys
import os
import argparse
from audiomass import (
    __version__,
    __release__,
    __rls_name__,
    __prg_name__,
)
from audiomass.utils import whichcraft
from audiomass.dir_conversion import DirConvert
from audiomass.batch_conversion import BatchConvert


def on_folder(inputdir, outputdir):
    """
    Check for dirs
    """
    if os.path.isdir(inputdir):
        path_in = os.path.abspath(os.path.join(inputdir))
    else:
        sys.exit(f"\033[31;1mERROR:\033[0m "
                 f"Invalid input folder '{inputdir}'")

    if outputdir == '.':
        path_out = None
    else:
        if os.path.isdir(outputdir):
            path_out = os.path.abspath(os.path.join(outputdir))
        else:
            sys.exit(f"\033[31;1mERROR:\033[0m "
                     f"Invalid output folder '{outputdir}'")

    return path_in, path_out


def on_files(inputfiles, outputdir):
    """
    Check for input files and output dir

    """
    for fname in inputfiles:
        if not os.path.isfile(os.path.abspath(os.path.join(fname))):
            sys.exit(f"\033[31;1mERROR:\033[0m "
                     f"Invalid filename '{fname}'")

    if outputdir == '.':
        path_out = None
    else:
        if os.path.isdir(outputdir):
            path_out = os.path.abspath(os.path.join(outputdir))
        else:
            sys.exit(f"\033[31;1mERROR:\033[0m "
                     f"Invalid output folder '{outputdir}'")

    return inputfiles, path_out


def main():
    """
    args parsing using argparse module
    """
    parser = argparse.ArgumentParser(
                prog=__prg_name__,
                description=(f"\n\033[1m{__prg_name__}\033[0m - Wrapper "
                             f"for multiple audio conversion libraries.\n"),
                )

    parser.add_argument('--version',
                        help="Show the current version and exit",
                        action='version',
                        version=(f"{__rls_name__} - version {__version__} - "
                                 f"released {__release__}"),
                        )
    parser.add_argument('-c', '--check-requires',
                        help="List of installed or missing dependencies",
                        action="store_true",
                        )
    parser.add_argument('-d', '--directory',
                        metavar='INPUT_FOLDER',
                        type=str,
                        help=("Path to input folder. Converts a bunch "
                              "of audio files contained in a folder"),
                        action="store",
                        )
    parser.add_argument('-f', '--files',
                        type=str,
                        help=("Convert one or more audio files separated "
                              "by spaces, even with different formats"),
                        nargs='+',
                        metavar='..FILE ..FILE',
                        action="store",
                        )
    parser.add_argument("-o", "--output",
                        action="store",
                        type=str,
                        dest="output_folder",
                        help=("Path to output folder, default same "
                              "destination as input file"),
                        required=False,
                        default='.'
                        )
    args = parser.parse_args()

    if len(sys.argv) == 1:
        parser.error("Use at least one optional argument")

    if args.files and args.directory:
        parser.error("No action requested, add '--directory' "
                     "or '-files', not both")

    if args.check_requires:
        print(f"\n\033[1m{__rls_name__}\033[0m - "
              f"check of available audio libraries:")
        whichcraft()

    if args.directory:
        inp, out = on_folder(args.directory, args.output_folder)
        conv = DirConvert(inp, out)
        conv.prompt_to_output_format()
        conv.command_builder()
        conv.end_check()

    elif args.files:
        inp, out = on_files(args.files, args.output_folder)
        conv = BatchConvert(inp, out)
        conv.prompt_to_output_format()
        conv.command_builder()
        conv.end_check()

    return 0


if __name__ == '__main__':
    STATUS = main()
    sys.exit(STATUS)



# -*- coding: utf-8 -*-
"""
Name:         utils.py (module)
Porpose:      All the useful functions used by the program.
Writer:       Gianluca Pernigoto <jeanlucperni@gmail.com>
Copyright:    (c) Gianluca Pernigoto <jeanlucperni@gmail.com>
license:      GPL3
Rev:          Dec 21 2021
Code checker: flake8, pylint
"""
import subprocess
import sys
import os
from shutil import which
from audiomass.datastrings import msgdebug, msgcustom
from audiomass.comparisions import supported_formats, comparing
from audiomass.comparisions import text_menu


def whichcraft(arg=None):
    """
    Without *arg* checks for binaries in *listing*
    and print result. Otherwise accepts one argument
    and returns result of *which*: `None` if not exist
    or its executable path-name.

    """
    if not arg:
        listing = ['ffmpeg', 'flac', 'lame', 'oggdec', 'oggenc',
                   'shntool', 'mac']
        # listing = ['sox', 'wavpack']  # this are for futures implementations
        for required in listing:
            # if which(required):
            if which(required, mode=os.F_OK | os.X_OK, path=None):
                msgcustom(f"Check for: '{required}' ..Ok")
            else:
                msgcustom(f"Check for: '{required}' ..Not Installed")
        return None

    return which(str(arg), mode=os.F_OK | os.X_OK, path=None)


def show_format_menu(indexes):
    """
    print a text menu with audio format references
    currently supported.
    """
    menu = text_menu()
    setmenu = [menu[i] for i in range(len(menu)) if i not in set(indexes)]
    for outformat in setmenu:  # realizzazione menu di output
        msgcustom(f"{outformat}")


def get_codec_data(input_format, output_select):
    """
    Return an available audio format and its codec data
    """
    try:
        selection = int(output_select)
    except ValueError:
        return None

    if selection in supported_formats():
        # get lower case str(audio format):
        output_format = supported_formats().get(selection)[1]
        codec_data = comparing(f'{input_format} > {output_format}')
    else:
        return None

    return codec_data


def build_cmd(codec, bitrate, in_file, out_file):
    """
    Since each command associated with a type of codec appears
    to be different, the key of the 'command_dict' must be equal
    with the first value of the 'comparision.object_assignment'
    dictionary.

    This function Return a string with the command correctly formed.

    The accepted parameters are:

    - codec: key of the command_dict
    - bitrate: str('value')
    - in_file: str('/dirname/inputfilename.ext')
    - out_file: str('/dirname/outfilename.ext')

    """
    output_format = os.path.splitext(os.path.basename(out_file))[1]
    dirname = os.path.dirname(out_file)

    command_dict = {
        'flac': f'flac -V {bitrate} "{in_file}" -o "{out_file}"',
        'lame': f'lame --nohist {bitrate} "{in_file}" "{out_file}"',
        'lame --decode': f'lame --decode "{in_file}" "{out_file}"',
        'oggenc': f'oggenc {bitrate} "{in_file}" -o "{out_file}"',
        'mac': f'mac "{in_file}" "{out_file}" {bitrate}',
        'ffmpeg': f'ffmpeg -i "{in_file}" {bitrate} "{out_file}"',
        'oggdec': f'oggdec "{in_file}" -o "{out_file}"',
        'shntool': (f'shntool conv -o {output_format.split(".")[1]} '
                    f'-O always "{in_file}" -d "{dirname}"')
                    }

    return command_dict[codec]


def run_subprocess(command):
    """
    Run command using subprocess.run
    """
    interrupted = None
    # print(command) # uncomment for debug, and comment try clause
    try:
        subprocess.run(command, check=True, shell=True)

    except subprocess.CalledProcessError as error:
        sys.exit(msgdebug(err=error))

    except KeyboardInterrupt:
        interrupted = True

    if interrupted:
        sys.exit(msgdebug(head='\n\n', warn="KeyboardInterrupt !"))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Name:          setup.py
Porpose:       building and setup audiomass package
Platform:      Gnu/Linux, Unix
Writer:        jeanslack <jeanlucperni@gmail.com>
Copyright:     (c) 2015 jeanslack <jeanlucperni@gmail.com>
license:       GPL3
Rev:           July.25.2020, Dec 08 2021
Code checker: flake8, pylint
"""
from os import path
from setuptools import setup, find_packages
from audiomass import (
    __author__,
    __mail__,
    # __copyright__,
    __version__,
    # __release__,
    # __rls_name__,
    __prg_name__,
    __url__,
    __short_descript__,
    # __long_descript__,
    # __license__,
    __short_license__,
    )


def build():
    """build"""

    # ---- categorize with ----#
    classifiers = [
            'Development Status :: 4 - Beta',
            'Environment :: Console',
            'Intended Audience :: End Users/Desktop',
            'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
            'Natural Language :: English',
            'Operating System :: MacOS :: MacOS X',
            'Operating System :: POSIX',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
            'Programming Language :: Python :: 3.9',
            'Topic :: Multimedia :: Sound/Audio :: Conversion',
            'Topic :: Utilities',
            ]

    # get the package data
    data_files = [('share/man/man1', ['man/audiomass.1.gz']),
                  ('share/audiomass', ['AUTHORS',
                                       'BUGS',
                                       'CHANGELOG',
                                       'COPYING',
                                       'README.md',
                                       'TODO']),
                  ]

    here = path.abspath(path.dirname(__file__))

    with open(path.join(here, 'README.md'), 'r', encoding='utf8') as readme:
        long_descript = readme.read()

    # Setup
    setup(name=__prg_name__,
          version=__version__,
          description=__short_descript__,
          long_description=long_descript,
          long_description_content_type='text/markdown',
          author=__author__[0],
          author_email=__mail__,
          url=__url__,
          license=__short_license__,
          platforms=["Linux", "Unix", "MacOS"],
          packages=find_packages(),
          data_files=data_files,
          zip_safe=False,
          python_requires=">=3.6",
          entry_points={
              "console_scripts": ['audiomass = audiomass.__main__:main']},
          classifiers=classifiers,
          )


if __name__ == '__main__':
    build()



# -*- coding: utf-8 -*-
"""
Name:         datastrings.py (module)
Porpose:      module for cosmetic output command line
Writer:       Gianluca Pernigoto <jeanlucperni@gmail.com>
Copyright:    (c) 2015/2019 Gianluca Pernigoto <jeanlucperni@gmail.com>
license:      GPL3
Rev:          nov 22 2017, Dec 15 2017, Aug 8 2019, Dec 08 2021
Code checker: flake8, pylint
"""


def msgdebug(info=None, warn=None, err=None, head='', tail=''):
    """
    print debug messages
    """
    if info:
        print(f"{head}\033[34;1mINFO:\033[0m {info}{tail}")
    if warn:
        print(f"{head}\033[33;1mWARNING:\033[0m {warn}{tail}")
    if err:
        print(f"{head}\033[31;1mERROR:\033[0m {err}{tail}")


def msgcolor(head='', tail='', orange=None, green=None, green2=None):
    """
    print informative messages
    """
    if orange:
        print(f"{head}\033[33;7m{orange}\033[0m{tail}")

    if green:
        print(f"{head}\033[32;7m{green}\033[0m{tail}")

    if green2:
        print(f"{head}\033[36;7m{green2}\033[0m{tail}")


def msgend(done=None, abort=None):
    """
    Print status messages
    """
    if done:
        print("\n\033[1m..Finished!\033[0m\n")
    if abort:
        print("\n\033[1m..Abort!\033[0m\n")


def msgcustom(message):
    """
    Print any string messages
    """
    print(message)
